/***************************************************************************
 *   Copyright (C) 2009 by Robert Marmorstein                              *
 *   robert@narnia.homeunix.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "basket_plugin.h"
#include "global.h"

#include "basket_part.h"

#include <KontactInterface/Core>

EXPORT_KONTACT_PLUGIN(BasketPlugin, basket)

BasketPlugin::BasketPlugin(KontactInterface::Core *core, const QVariantList &)
        : KontactInterface::Plugin(core, core, "Basket")
{
    setComponentData(KontactPluginFactory::componentData());
    Global::basketConfig = KSharedConfig::openConfig("basketrc");
}

BasketPlugin::~BasketPlugin()
{
}

KParts::ReadOnlyPart *BasketPlugin::createPart()
{
    KParts::ReadOnlyPart *part = loadPart();

    connect(part, SIGNAL(showPart()), this, SLOT(showPart()));

    return part;
}

void BasketPlugin::readProperties(const KConfigGroup &config)
{
    if (part()) {
        BasketPart *myPart = static_cast<BasketPart*>(part());
    }
}

void BasketPlugin::saveProperties(KConfigGroup &config)
{
    if (part()) {
        BasketPart *myPart = static_cast<BasketPart*>(part());
    }
}

void BasketPlugin::showPart()
{
    core()->selectPlugin(this);
}
